function [SINR_mvdr,SINR_smi] = mvdr(snr_db,snapshots,N,K);

% This code simulates MVDR beamforming algorithms for a uniform linear
% array (ULA) and outputs the signal-to-interference-plus-noise ratio (SINR)
% over a number of snapshots
% d                                   % inter-element spacing of ULA.
% K                                   % number of users in the system (including one desired user)
% N                                   % number of sensor elements
% snapshots                           % total number of snapshots
%
% Author: Rodrigo C. de Lamare, 2000

%Antenna parameters:
d = 0.5; % spacing of elements in terms of wavelength

%Parameters:
delta = 0.01;            % regularisation coefficient

% Directions of arrival of the desired signal and the interferers
phase = linspace(30, 180, K); 

% Steering vectors for ULA
A = zeros(N,K);
for n=1:N
    A(n,:) = exp(-pi*j*d*(n-1)*sin(phase*pi/180));
end
A = A/sqrt(N); % normalization
a_d = A(:,1); % steering vector of desired signal (assumed to be the first signal)

% Amplitude of the received signals 
a_s = ones(K,1);
a_s(2:K) = 1; % power level of interferers

% Received signals (BPSK)
%s = a_s.*sign(rand(K,1)-0.5);
s = a_s.*randn(K,1); 

% Noise standard deviation in relation to desired signal
snr = exp(snr_db*log(10)/10);
s_n = a_s(1)/sqrt(snr);                       


% Covariance matrices (assuming unit signal power)
I = eye(N);
R = A*diag(a_s)*diag(a_s)*A' + s_n^2*I;
R_inv = inv(R);
R_s = A(:,1)*a_s(1)*a_s(1)*A(:,1)'; % desired signal covariance matrix
R_i = A(:,2:K)*diag(a_s(2:K))*diag(a_s(2:K))*A(:,2:K)' + s_n^2*I; % signal to interference plus noise covariance matrix

% Beamformers
w_smi = zeros(N,1); w_smi(1) = 0.1; 
k = zeros(N,1); P = 0;% delta*I;
w_mvdr = zeros(N,1); w_mvdr = inv(a_d'*R_inv*a_d)*R_inv*a_d;

% SINR vectors
SINR_mvdr = zeros(snapshots,1);
SINR_smi = zeros(snapshots,1);


for i = 1:snapshots
    % noise at receiver
    n = sqrt(s_n^2/2)*randn(N,1) + j*sqrt(s_n^2/2)*randn(N,1);
    % source signals 
    %s = a_s.*sign(rand(K,1)-0.5);
    s = a_s.*randn(K,1);
    % received data
    r = A*s + n;
    % MVDR
    y_mvdr = w_mvdr'*r;
    % SMI algorithm
    P = P + r*r';
    P_inv = pinv(P/i);
    w_smi = P_inv*a_d/(a_d'*P_inv*a_d);
    y_smi = w_smi'*r;
    % SINR values
    SINR_mvdr(i) = (w_mvdr'*R_s*w_mvdr)/(w_mvdr'*R_i*w_mvdr);
    SINR_smi(i) = (w_smi'*R_s*w_smi)/(w_smi'*R_i*w_smi);
end

    










